SystemOrganization addCategory: #Roassal2!

TestCase subclass: #RTTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Roassal2'!

RTTest subclass: #RTEdgeTest
	instanceVariableNames: 'el1 el2 edge'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Roassal2'!

!RTEdgeTest methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/16/2013 17:59'!
setUp
	el1 := RTBox element.
	el2 := RTBox element.
	el1 translateTo: 20 @ 20.
	el2 translateTo: 60 @ 80.
	edge := RTLine edgeFrom: el1 to: el2! !

!RTEdgeTest methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/16/2013 17:59'!
testBasic

	RTView new
		add: el1;
		add: el2;
		add: edge;
		createAndFillCanvas.
	self assert: edge trachelShape notNil.
	self assert: edge trachelShape color notNil! !

!RTEdgeTest methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/16/2013 19:07'!
testIncomingAndOutgoingEdges

	self assert: (el1 outgoingEdges includes: edge).
	self assert: (el2 incomingEdges includes: edge).! !

!RTEdgeTest methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/16/2013 19:20'!
testTranslatingNode
	| old |
	RTView new
		add: el1;
		add: el2;
		add: edge;
		createAndFillCanvas.

	old := edge trachelShape from.
	self assert: edge trachelShape from = el1 position.
	self assert: edge trachelShape to = el2 position.
	
	el1 translateBy: 10 @ 5.
	self assert: edge trachelShape from = (old + (10 @ 5)).
	self assert: edge trachelShape from = el1 position.
	self assert: edge trachelShape to = el2 position.! !

RTTest subclass: #RTElementTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Roassal2'!

!RTElementTest methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/14/2013 22:24'!
testDefault
	| el |
	el := RTElement new.
	self assert: el position = (0 @ 0).
	self assert: el width = 0.
	self assert: el height = 0.! !

!RTElementTest methodsFor: 'interaction' stamp: 'AlexandreBergel 12/18/2013 14:11'!
testDraggingCircle

	| view shape el |
	view := RTView new.
	shape := RTEllipse new size: 40.
	el := shape element.
	view add: el.
	view createAndFillCanvas.
	
	self assert: el position = (0 @ 0).
	el translateBy: 100 @ 50.
	self assert: el position = (100 @ 50).
	
	self assert: el trachelShape rectangle = ((80@30) corner: (120@70))! !

!RTElementTest methodsFor: 'interaction' stamp: 'AlexandreBergel 12/16/2013 15:15'!
testHighlighting
	| el view |
	el := RTBox element.
	view := RTView new.
	view add: el.
	view createAndFillCanvas.
	
	self assert: el trachelShape color = RTBox new defaultColor.
	
	el trachelShape color: Color red.
	self assert: el trachelShape color = Color red.! !

!RTElementTest methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/14/2013 21:10'!
testModel
	self assert: (RTElement on: 10) model = 10! !

!RTElementTest methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/16/2013 13:06'!
testRendering

	| view el |
	view := RTView new.
	el := RTBox element.
	view add: el.
	
	self assert: el trachelShape isNil.
	view createAndFillCanvas.
	self assert: el trachelShape notNil.
	self assert: el trachelShape class == TRBoxShape! !

!RTElementTest methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/16/2013 13:09'!
testRenderingAndEvent

	| view el trachelShape |
	view := RTView new.
	el := RTBox element.
	view add: el.
	view createAndFillCanvas.
	trachelShape := el trachelShape.
	self assert: el announcer == trachelShape announcer! !

!RTElementTest methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/14/2013 21:20'!
testSameNullShape
	self assert: RTElement new shape == RTElement new shape.
	self assert: RTNull element shape == RTElement new shape.! !

!RTElementTest methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/14/2013 22:26'!
testSize
	| box el |
	box := RTBox new width: 10; height: 8.
	el := box element.
	
	self assert: el position = (0 @ 0).
	self assert: el height = 8.
	self assert: el width = 10! !

!RTElementTest methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/14/2013 22:47'!
testSizeAndModel
	| box el |
	box := RTBox new width: [ :e | e model + 2 ]; height: [ :e | e model + 3 ].
	el := box elementOn: 5.
	
	self assert: el position = (0 @ 0).
	self assert: el height = 8.
	self assert: el width = 7! !

!RTElementTest methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/14/2013 21:05'!
testTranslationBy
	| el |
	el := RTElement new.
	el translateBy: 15 @ 10.
	self assert: el position = (15 @ 10).

	el translateBy: 5 @ 10.
	self assert: el position = (20 @ 20).
! !

!RTElementTest methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/14/2013 21:05'!
testTranslationTo
	| el |
	el := RTElement new.
	el translateTo: 15 @ 10.
	self assert: el position = (15 @ 10)! !

RTTest subclass: #RTLayoutTest
	instanceVariableNames: 'view'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Roassal2'!

!RTLayoutTest methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/14/2013 22:18'!
setUp
	super setUp.
	view := RTView new.
	
	30 timesRepeat: [ view add: RTBox element ]! !

!RTLayoutTest methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/14/2013 22:18'!
test

	RTHorizontalLineLayout on: view elements! !

RTTest subclass: #RTShapeTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Roassal2'!

!RTShapeTest methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/16/2013 16:43'!
testColoredShape

	| shape el |
	shape := RTBox new color: Color red.
	el := shape element.
	RTView new add: el; createAndFillCanvas.
	self assert: el trachelShape color = Color red! !

!RTShapeTest methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/14/2013 21:57'!
testElementCreation

	| s |
	s := RTBox new.
	self assert: s element shape == s! !

!RTShapeTest methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/14/2013 21:41'!
testInstance

	| s |
	self assert: RTBox instance class == RTBox.
	self assert: RTBox instance ~~ RTBox instance.
	
	s := RTBox instance.
	self assert: s instance == s.! !

!RTShapeTest methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/14/2013 21:39'!
testShapeCreation

	self assert: RTBox element shape class == RTBox! !

RTTest subclass: #RTShapedObjectTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Roassal2'!

!RTShapedObjectTest methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/16/2013 17:10'!
testAddingInteraction

	| el |
	el := RTBox element.
	el @ RTHighlightable.
	
	RTView new add: el; createAndFillCanvas.

	self assert: el trachelShape color = RTBox new color.
	el announce: TRMouseEnter.
	self assert: el trachelShape color = RTHighlightable highlightedColor.! !

!RTShapedObjectTest methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/16/2013 10:04'!
testEmittingEvent

	| el t |
	t := 0.
	el := RTBox element.
	el on: TRMouseEnter do: [ :evt | t := t + 1 ].
	
	self assert: t = 0.
	el announce: TRMouseEnter.
	self assert: t = 1
	! !

RTTest subclass: #RTViewTest
	instanceVariableNames: 'view'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Roassal2'!

!RTViewTest methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/14/2013 20:37'!
setUp
	super setUp.
	view := RTView new! !

!RTViewTest methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/14/2013 20:38'!
testAdding

	view add: RTElement new.
	self assert: view numberOfElements = 1! !

!RTViewTest methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/14/2013 20:38'!
testDefault

	self assert: view numberOfElements = 0! !

!RTViewTest methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/16/2013 11:24'!
testEmptyCanvas
	self assert: RTView new createAndFillCanvas numberOfShapes = 0
! !

!RTViewTest methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/16/2013 11:27'!
testFillingCanvas

	| canvas |
	view := RTView new.
	view add: RTBox element.
	canvas := view createAndFillCanvas.
	
	self assert: canvas class == TRCanvas.
	self assert: canvas numberOfShapes = 1.! !

!BlockClosure methodsFor: '*Roassal2-Core' stamp: 'AlexandreBergel 12/14/2013 22:32'!
rtValue: valueOrArray
	self numArgs = 0 ifTrue: [ ^ self value ].  
	self numArgs = 1 ifTrue: [ ^ self value: valueOrArray ].
	
	"if I have more than 1 argument, then the valueOrArray is a collection"
	valueOrArray isCollection ifFalse: [ self error: 'A block with more than one argument can only be evaluated with a collection ' ].
	
	self numArgs = valueOrArray size ifTrue: [ ^ self valueWithArguments: valueOrArray ]. 
	
	self error: 'Incorrect number of arguments'
! !

!Object methodsFor: '*Roassal2' stamp: 'AlexandreBergel 12/14/2013 21:45'!
rtValue: anArgument 
	^ self! !

Object subclass: #RTObject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Roassal2'!

RTObject subclass: #RTExample
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Roassal2'!

!RTExample methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/14/2013 21:53'!
basic
	"
	self new basic
	"
	
	| view |
	view := RTView new.
	view add: (RTBox element translateTo: 20 @ 20; yourself).
	view open! !

!RTExample methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/18/2013 12:56'!
circles
	"
	self new circles
	"

	| view shape el |
	view := RTView new.
	shape := RTEllipse new color: (Color purple alpha: 0.2); size: 80.
	100 timesRepeat: [ 
		el := shape element translateTo: 500 atRandom @ 500 atRandom.
		el @ RTDraggable.
		view add: el. 
	].	
	view open! !

!RTExample methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/14/2013 22:51'!
growingSquares
	"
	self new growingSquares
	"
	
	| view shape |
	view := RTView new.

	shape := RTBox new size: [ :e | e model yourself ].
	1 to: 100 do: [ :i |
		view add: ((shape elementOn: i) translateTo: 20 @ 20; yourself).
	].
	RTGridLayout on: view elements.
	
	view open! !

!RTExample methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/16/2013 15:56'!
highlightSquare
	"
	self new highlightSquare
	"
	
	| view shape el |
	view := RTView new.

	shape := RTBox new size: [ :e | e model yourself ].
	1 to: 100 do: [ :i |
		el := shape elementOn: i.
		el translateTo: 20 @ 20.
		el on: TRMouseEnter do: [ :e | e shape color: Color red. e signalUpdate ].
		el on: TRMouseLeave do: [ :e | e shape color: Color gray. e signalUpdate ].
		el on: TRMouseDragging do: [ :e | e shape translateBy: e step. e signalUpdate ].
		view add: el.
	].
	RTGridLayout on: view elements.
	
	view open! !

!RTExample methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/17/2013 14:41'!
linkedSquares
	"
	self new linkedSquares
	"
	| view shape  fromEl toEl edge |
	view := RTView new.

	shape := RTBox new size: 90; color: (Color blue alpha: 0.1).
	fromEl := shape element.
	fromEl translateTo: 20 @ 30.
	fromEl @ RTDraggable.

	toEl := shape element.
	toEl translateTo: 200 @ 80.
	toEl @ RTDraggable.
	
	edge := RTLine edgeFrom: fromEl to: toEl.  	
	view add: fromEl; add: toEl; add: edge.

	view open.
	view inspect! !

!RTExample methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/16/2013 17:15'!
manySquares
	"
	self new manySquares
	"
	
	| view shape el |
	view := RTView new.

	shape := RTBox new.
	1 to: 7000 do: [ :i |
		el := shape elementOn: i.
		el translateTo: 20 @ 20.
		el @ RTHighlightable.
		el @ RTDraggable.
		view add: el.
	].
	RTGridLayout new gapSize: 1; on: view elements.
	
	view open! !

RTObject subclass: #RTInteraction
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Roassal2'!

!RTInteraction commentStamp: '' prior: 0!
A ROInteraction is xxxxxxxxx.
!

RTInteraction subclass: #RTDraggable
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Roassal2'!

!RTDraggable methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/18/2013 14:12'!
initializeElement: element
	element on: TRMouseDragging do: [ :e | Halt once. e element translateBy: e step. e signalUpdate ]! !

RTInteraction subclass: #RTHighlightable
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Roassal2'!

RTHighlightable class
	instanceVariableNames: 'lastHighlightedShape lastColor'!

RTHighlightable class
	instanceVariableNames: 'lastHighlightedShape lastColor'!

!RTHighlightable class methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/16/2013 17:10'!
highlightedColor
	^ Color blue! !

!RTHighlightable class methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/16/2013 17:00'!
lastColor
	^ lastColor! !

!RTHighlightable class methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/16/2013 17:00'!
lastColor: aColor
	lastColor := aColor! !

!RTHighlightable class methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/16/2013 16:59'!
lastHighlightedShape
	^ lastHighlightedShape! !

!RTHighlightable class methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/16/2013 17:00'!
lastHighlightedShape: aTrachelShape
	lastHighlightedShape := aTrachelShape! !

!RTHighlightable methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/16/2013 17:10'!
highlightedColor
	^ self class highlightedColor! !

!RTHighlightable methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/16/2013 17:09'!
initializeElement: element
	element
		on: TRMouseEnter
		do: [ :e | 
			self unhighlightRecordedShape.
			self recordShape: e shape.
			e shape color: self highlightedColor.
			e signalUpdate ].
	element
		on: TRMouseLeave
		do: [ :e | 
			self unhighlightRecordedShape.
			e signalUpdate ]! !

!RTHighlightable methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/16/2013 17:05'!
recordShape: aShape
	self class lastHighlightedShape: aShape.
	self class lastColor: aShape color
! !

!RTHighlightable methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/16/2013 17:06'!
unhighlightRecordedShape
	self class lastHighlightedShape isNil ifTrue: [ ^ self ].
	self class lastHighlightedShape color: self class lastColor.
	self class lastHighlightedShape: nil! !

!RTInteraction class methodsFor: 'public' stamp: 'AlexandreBergel 12/16/2013 16:57'!
initializeElement: element
	^ self new initializeElement: element; yourself! !

!RTInteraction methodsFor: 'hooks' stamp: 'AlexandreBergel 12/16/2013 16:54'!
initializeElement: element
	self subclassResponsibility ! !

RTObject subclass: #RTLayout
	instanceVariableNames: 'affectedNodes translator eventHandler currentIteraction maxInterations iterationsToSendEvent paddingLeft paddingRight paddingTop'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Roassal2'!

!RTLayout commentStamp: '<historical>' prior: 0!
A ROLayout is the superclass of all.

Instance Variables
	affectedNodes:		<Object>
	currentIteraction:		<Object>
	eventHandler:		<Object>
	maxInterations:		<Object>
	translator:		<Object>

affectedNodes
	- xxxxx

currentIteraction
	- xxxxx

eventHandler
	- xxxxx

maxInterations
	- xxxxx

translator
	- xxxxx
!

RTLayout subclass: #RTAbstractGridLayout
	instanceVariableNames: 'gapSize lineItemsCountBlock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Roassal2'!

!RTAbstractGridLayout commentStamp: '' prior: 0!
A ROAbstractGridLayout is xxxxxxxxx.

Instance Variables
	gapSize:		<Object>
	lineItemsCountBlock:		<Object>

gapSize
	- xxxxx

lineItemsCountBlock
	- xxxxx
!

!RTAbstractGridLayout class methodsFor: 'public' stamp: 'AlexandreBergel 12/14/2013 22:50'!
isAbstract
	^ self name =  #ROAbstractGridLayout! !

!RTAbstractGridLayout class methodsFor: 'public' stamp: 'AlexandreBergel 12/14/2013 22:50'!
on: aCollectionOfElements withGap: anInteger withLineItemsCount: aBlock
	"place the elements in a grid with (aBlock roValue: aCollectionOfElements) as the amount of elements horizontally"
	
	| myLayout |
	myLayout := self new gapSize: anInteger;  lineItemsCount: aBlock; yourself.
	myLayout applyOn: aCollectionOfElements.
	^ aCollectionOfElements! !

!RTAbstractGridLayout class methodsFor: 'public' stamp: 'AlexandreBergel 12/14/2013 22:50'!
on: aCollectionOfElements withLineItemsCount: aBlock
	"place the elements in a grid with (aBlock roValue: aCollectionOfElements) as the amount of elements horizontally

	For example:
	self on: (ROElement forCollection: (1 to: 20)) withLineItemsCount: 5
	=> place the 20 elements on a grid 5 x 4
	
	self on: (ROElement forCollection: (1 to: 20)) withLineItemsCount: [ :elements | elements size // 3 ]
	=> place the 20 elements on a grid 3 x 7
	"
	
	
	| myLayout |
	myLayout := self new lineItemsCount: aBlock; yourself.
	myLayout applyOn: aCollectionOfElements.
	^ aCollectionOfElements! !

!RTAbstractGridLayout class methodsFor: 'instance creation' stamp: 'AlexandreBergel 12/14/2013 22:50'!
withGap: anInteger 

	^ self new 
			gapSize: anInteger; 
			yourself! !

!RTAbstractGridLayout class methodsFor: 'instance creation' stamp: 'AlexandreBergel 12/14/2013 22:50'!
withGap: anInteger withLineItemsCount: aBlock
	
	^self new
		gapSize: anInteger;
		lineItemsCount: aBlock;
		yourself! !

!RTAbstractGridLayout class methodsFor: 'instance creation' stamp: 'AlexandreBergel 12/14/2013 22:50'!
withLineItemsCount: aBlock
	
	^self new
		lineItemsCount: aBlock;
		yourself! !

!RTAbstractGridLayout methodsFor: 'initialize-release' stamp: 'AlexandreBergel 12/14/2013 22:50'!
defaultGapSize
	^ 5! !

!RTAbstractGridLayout methodsFor: 'accessing' stamp: 'AlexandreBergel 12/14/2013 22:50'!
defaultLineItemsCount
	
	^ [ :elements | 
			| height width |
			(elements size < 3 
				ifTrue: [ (elements size max: 1) @ 1 ]
				ifFalse: 
					[ height := (elements size * 0.618034) sqrt ceiling truncated.
					width := (elements size / height) ceiling truncated.
					width @ height ]) x ]! !

!RTAbstractGridLayout methodsFor: 'accessing' stamp: 'AlexandreBergel 12/14/2013 22:50'!
gapSize
	
	^gapSize! !

!RTAbstractGridLayout methodsFor: 'accessing' stamp: 'AlexandreBergel 12/14/2013 22:50'!
gapSize: anInteger
	gapSize := anInteger! !

!RTAbstractGridLayout methodsFor: 'initialize-release' stamp: 'AlexandreBergel 12/14/2013 22:50'!
initialize
	super initialize.
	gapSize := self defaultGapSize.
	lineItemsCountBlock := self defaultLineItemsCount.! !

!RTAbstractGridLayout methodsFor: 'accessing' stamp: 'AlexandreBergel 12/14/2013 22:50'!
lineItemsCount: aBlock
	
	lineItemsCountBlock := aBlock! !

!RTAbstractGridLayout methodsFor: 'accessing' stamp: 'AlexandreBergel 12/14/2013 22:50'!
lineItemsCountBlock
	
	^ lineItemsCountBlock! !

RTAbstractGridLayout subclass: #RTGridLayout
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Roassal2'!

!RTGridLayout commentStamp: '<historical>' prior: 0!
A ROGridLayout places elements as a grid.

Instance Variables
	gapSize:		<SmallInteger>
	lineItemsCountBlock:		<BlockContext>

gapSize
	- number of pixels between each elements, horizontally and vertically

lineItemsCountBlock
	- tells the amount of item per line should be used
!

!RTGridLayout methodsFor: 'hook' stamp: 'AlexandreBergel 12/14/2013 22:52'!
doExecute: elements
	| pointer lineItemCount lineItemSize maxLastLineHeight originalGapLeft originalGapTop |
	originalGapLeft := self paddingLeftFor: elements.
	originalGapTop := self paddingTopFor: elements.
	
	pointer := originalGapLeft @ originalGapTop.
	lineItemSize := self lineItemsCountBlock roValue: elements.
	lineItemCount := 0.
	maxLastLineHeight := 0.

	elements
		do: [ :element | 
			translator translate: element to: pointer.
			pointer := (pointer x + element width + (self gapSize * 2)) @ pointer y.
			lineItemCount := lineItemCount + 1.
			maxLastLineHeight := maxLastLineHeight max: element height.
			lineItemCount >= lineItemSize
				ifTrue: [ 
					pointer := originalGapLeft @ (pointer y + (self gapSize * 2) + maxLastLineHeight).
					
					maxLastLineHeight := 0.
					lineItemCount := 0 ].
			self step ].
! !

RTLayout subclass: #RTAbstractLineLayout
	instanceVariableNames: 'gapSize horizontalGap verticalGap horizontalOutGap verticalOutGap alignment horizontallyStretchable verticallyStretchable'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Roassal2'!

!RTAbstractLineLayout commentStamp: '<historical>' prior: 0!
A ROAbstractLineLayout is the abstract superclass of the line layout.

Instance Variables
	alignment:		<Object>
	gapSize:		<Object>
	horizontalGap:		<Object>
	horizontalOutGap:		<Object>
	horizontallyStretchable:		<Object>
	verticalGap:		<Object>
	verticalOutGap:		<Object>
	verticallyStretchable:		<Object>

alignment
	- xxxxx

gapSize
	- xxxxx

horizontalGap
	- xxxxx

horizontalOutGap
	- xxxxx

horizontallyStretchable
	- xxxxx

verticalGap
	- xxxxx

verticalOutGap
	- xxxxx

verticallyStretchable
	- xxxxx
!

!RTAbstractLineLayout class methodsFor: 'testing' stamp: 'AlexandreBergel 12/14/2013 22:05'!
isAbstract
	^ self name =  #ROAbstractLineLayout! !

!RTAbstractLineLayout class methodsFor: 'instance creation' stamp: 'AlexandreBergel 12/14/2013 22:05'!
withGap: anInteger
	
	^(self new)
		gapSize: anInteger;
		yourself! !

!RTAbstractLineLayout methodsFor: 'accessing' stamp: 'AlexandreBergel 12/14/2013 22:05'!
alignBottom
	
	alignment := #bottom! !

!RTAbstractLineLayout methodsFor: 'accessing' stamp: 'AlexandreBergel 12/14/2013 22:05'!
alignCenter
	
	alignment := #center! !

!RTAbstractLineLayout methodsFor: 'accessing' stamp: 'AlexandreBergel 12/14/2013 22:05'!
alignLeft
	
	alignment := #left! !

!RTAbstractLineLayout methodsFor: 'accessing' stamp: 'AlexandreBergel 12/14/2013 22:05'!
alignRight
	
	alignment := #right! !

!RTAbstractLineLayout methodsFor: 'accessing' stamp: 'AlexandreBergel 12/14/2013 22:05'!
alignTop
	
	alignment := #top! !

!RTAbstractLineLayout methodsFor: 'accessing' stamp: 'AlexandreBergel 12/14/2013 22:05'!
alignment
	
	^alignment! !

!RTAbstractLineLayout methodsFor: 'accessing' stamp: 'AlexandreBergel 12/14/2013 22:05'!
alignment: anObject
	
	alignment := anObject! !

!RTAbstractLineLayout methodsFor: 'configuration' stamp: 'AlexandreBergel 12/14/2013 22:05'!
center
	self alignment: #center! !

!RTAbstractLineLayout methodsFor: 'hook' stamp: 'AlexandreBergel 12/14/2013 22:05'!
deltaFor: aNodeFigure
	
	^self subclassResponsibility! !

!RTAbstractLineLayout methodsFor: 'hook' stamp: 'AlexandreBergel 12/14/2013 22:05'!
doCenter: aGraph
	self subclassResponsibility ! !

!RTAbstractLineLayout methodsFor: 'hook' stamp: 'AlexandreBergel 12/14/2013 22:05'!
doExecute: elements 
	| pointer delta |
	horizontallyStretchable ifTrue: 
		[ self doStretchHorizontal: elements ].
	verticallyStretchable ifTrue: 
		[ self doStretchVertical: elements ].
	pointer := self positionOriginalPointer: elements.
	elements do: 
		[ :element |
		delta := self deltaFor: element.
		translator translate: element to: pointer - delta.
		pointer := self 
					movePointer: pointer
					accordingToFigure: element.
		self step ].! !

!RTAbstractLineLayout methodsFor: 'hook' stamp: 'AlexandreBergel 12/14/2013 22:05'!
doStretchHorizontal: aCollectionOfElements 

	self subclassResponsibility! !

!RTAbstractLineLayout methodsFor: 'hook' stamp: 'AlexandreBergel 12/14/2013 22:05'!
doStretchVertical: aCollectionOfElements 

	self subclassResponsibility! !

!RTAbstractLineLayout methodsFor: 'accessing' stamp: 'AlexandreBergel 12/14/2013 22:05'!
gapSize
	
	^gapSize! !

!RTAbstractLineLayout methodsFor: 'accessing' stamp: 'AlexandreBergel 12/14/2013 22:05'!
gapSize: aNumber
	gapSize := aNumber. " This value is never used actually "
	
	verticalGap := 2 * aNumber.
	horizontalGap := 2 * aNumber! !

!RTAbstractLineLayout methodsFor: 'accessing' stamp: 'AlexandreBergel 12/14/2013 22:05'!
horizontalGap
	
	^horizontalGap! !

!RTAbstractLineLayout methodsFor: 'accessing' stamp: 'AlexandreBergel 12/14/2013 22:05'!
horizontalGap: anObject
	
	horizontalGap := anObject! !

!RTAbstractLineLayout methodsFor: 'testing' stamp: 'AlexandreBergel 12/14/2013 22:05'!
horizontallyStretchable
	^ horizontallyStretchable! !

!RTAbstractLineLayout methodsFor: 'initialize-release' stamp: 'AlexandreBergel 12/14/2013 22:05'!
initialize
	
	super initialize.
	verticalGap := 10.
	horizontalGap := 10.
	horizontallyStretchable := false.
	verticallyStretchable := false! !

!RTAbstractLineLayout methodsFor: 'testing' stamp: 'AlexandreBergel 12/14/2013 22:05'!
isLineLayout
	^ true! !

!RTAbstractLineLayout methodsFor: 'hook' stamp: 'AlexandreBergel 12/14/2013 22:05'!
movePointer: pointer accordingToFigure: aNodeFigure
	
	self subclassResponsibility! !

!RTAbstractLineLayout methodsFor: 'hook' stamp: 'AlexandreBergel 12/14/2013 22:05'!
positionOriginalPointer: aGraph
	
	^self subclassResponsibility! !

!RTAbstractLineLayout methodsFor: 'configuration' stamp: 'AlexandreBergel 12/14/2013 22:05'!
stretch
	horizontallyStretchable := verticallyStretchable := true! !

!RTAbstractLineLayout methodsFor: 'configuration' stamp: 'AlexandreBergel 12/14/2013 22:05'!
stretchHorizontally
	horizontallyStretchable := true! !

!RTAbstractLineLayout methodsFor: 'configuration' stamp: 'AlexandreBergel 12/14/2013 22:05'!
stretchVertically
	verticallyStretchable := true! !

!RTAbstractLineLayout methodsFor: 'accessing' stamp: 'AlexandreBergel 12/14/2013 22:05'!
verticalGap
	
	^verticalGap! !

!RTAbstractLineLayout methodsFor: 'accessing' stamp: 'AlexandreBergel 12/14/2013 22:05'!
verticalGap: anObject
	
	verticalGap := anObject! !

!RTAbstractLineLayout methodsFor: 'testing' stamp: 'AlexandreBergel 12/14/2013 22:05'!
verticallyStretchable
	^ verticallyStretchable! !

RTAbstractLineLayout subclass: #RTHorizontalLineLayout
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Roassal2'!

!RTHorizontalLineLayout commentStamp: '<historical>' prior: 0!
A RTHorizontalLineLayout locates all the elements horizontally!

!RTHorizontalLineLayout methodsFor: 'hook' stamp: 'AlexandreBergel 12/14/2013 22:07'!
deltaFor: aNodeFigure
	
	| delta |
	delta := 0.
	self alignment == #bottom ifTrue: [delta := aNodeFigure height].
	self alignment == #center ifTrue: [delta := aNodeFigure height / 2.0].
	^0 @ delta! !

!RTHorizontalLineLayout methodsFor: 'hook' stamp: 'AlexandreBergel 12/14/2013 22:07'!
doCenter: elements 

 	| midTallest step |
	midTallest := 0.
	midTallest := elements nodes 
		inject: 0
		into: [ :m :el | m max: (el bounds height )].
	midTallest := midTallest / 2.
	elements nodes do: [ :node | 
		step := 0 @ (midTallest - (node bounds height / 2)) asInteger.
		node bounds origin: (node bounds origin + step).
		node bounds corner: (node bounds corner + step).
	] 
! !

!RTHorizontalLineLayout methodsFor: 'hook' stamp: 'AlexandreBergel 12/14/2013 22:07'!
doStretchHorizontal: aCollectionOfElements 

 	| parent parentBounds addedWidth parentBoundsWidth runningIndex newWidth |

	aCollectionOfElements isEmpty ifTrue: [ ^ self "nothing to do" ].
	parent := aCollectionOfElements anyOne parent.
	parent isView ifTrue: [ ^ self "ROView does not have bounds" ].
	parentBounds := parent bounds.

	addedWidth := aCollectionOfElements
		inject: 0
		into: [ :m :el | m + el width ].

	parentBoundsWidth := parentBounds width.
"	parentBoundsWidth := parentBoundsWidth - ((aCollectionOfElements size - 1) * horizontalGap) - (2 * horizontalOutGap)."
	
	
	runningIndex := 0.
	parentBounds width > addedWidth
		ifTrue: [ aCollectionOfElements do: [ :element | 
						newWidth := (element width * parentBoundsWidth / addedWidth) asInteger.
						element width: newWidth.
						runningIndex := runningIndex + newWidth + horizontalGap ] ]! !

!RTHorizontalLineLayout methodsFor: 'hook' stamp: 'AlexandreBergel 12/14/2013 22:07'!
doStretchVertical: aCollectionOfElements 

 	| parent parentBounds |
	aCollectionOfElements isEmpty ifTrue: [ ^ self "nothing to do" ].
	parent := aCollectionOfElements anyOne parent.
	parent isView ifTrue: [ ^ self "ROView does not have bounds" ].
	parentBounds := parent bounds.

	aCollectionOfElements do: [ :element | 
		element height: (parentBounds height) ]! !

!RTHorizontalLineLayout methodsFor: 'initialize-release' stamp: 'AlexandreBergel 12/14/2013 22:07'!
initialize
	
	super initialize.
	self alignTop! !

!RTHorizontalLineLayout methodsFor: 'hook' stamp: 'AlexandreBergel 12/14/2013 22:07'!
movePointer: pointer accordingToFigure: element
	
	^ (pointer x + element width + self horizontalGap) @ pointer y! !

!RTHorizontalLineLayout methodsFor: 'hook' stamp: 'AlexandreBergel 12/14/2013 22:07'!
positionOriginalPointer: elements 
	| maxHeight delta |
	delta := 0.
	self alignment == #bottom ifTrue: 
		[ maxHeight := elements maxValue: #height.
		delta := maxHeight ].
	self alignment == #center ifTrue: 
		[ maxHeight := elements maxValue: #height.
		delta := maxHeight / 2.0 ].
	^ (self paddingLeftFor: elements) @ ((self paddingTopFor: elements) + delta)! !

RTAbstractLineLayout subclass: #RTVerticalLineLayout
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Roassal2'!

!RTVerticalLineLayout commentStamp: '<historical>' prior: 0!
A RTVerticalLineLayout locates all the elements vertically!

!RTVerticalLineLayout methodsFor: 'hook' stamp: 'AlexandreBergel 12/14/2013 22:06'!
deltaFor: aNodeFigure
	
	| delta |
	delta := 0.
	self alignment == #right ifTrue: [delta := aNodeFigure width].
	self alignment == #center ifTrue: [delta := aNodeFigure width / 2.0].
	^delta @ 0! !

!RTVerticalLineLayout methodsFor: 'hook' stamp: 'AlexandreBergel 12/14/2013 22:06'!
doCenter: aGraph 

 	| midWidest step |
	midWidest := aGraph nodes 
		inject: 0
		into: [ :m :el | m max: (el bounds width )].
	midWidest := midWidest / 2.
	aGraph nodes do: [ :node | 
		step := (midWidest - (node bounds width /2)) asInteger @ 0.
		node bounds origin: (node bounds origin + step).
		node bounds corner: (node bounds corner + step).
	] 
! !

!RTVerticalLineLayout methodsFor: 'hook' stamp: 'AlexandreBergel 12/14/2013 22:06'!
doStretchHorizontal: aCollectionOfElements 
	
 	| parent parentBounds |
	aCollectionOfElements isNil ifTrue: [ ^ self "nothing to do" ].
	parent := aCollectionOfElements anyOne parent.
	parent isView ifTrue: [ ^ self "ROView does not have bounds" ].
	parentBounds := parent bounds.

	aCollectionOfElements do: [ :element | 
		element width: parentBounds width ]! !

!RTVerticalLineLayout methodsFor: 'hook' stamp: 'AlexandreBergel 12/14/2013 22:06'!
doStretchVertical: aCollectionOfElements 

 	| parent parentBounds addedHeight parentBoundsHeight runningIndex newHeight |

	aCollectionOfElements isEmpty ifTrue: [ ^ self "nothing to do" ].
	parent := aCollectionOfElements anyOne parent.
	parent isView ifTrue: [ ^ self "ROView does not have bounds" ].
	parentBounds := parent bounds.

	addedHeight := aCollectionOfElements 
		inject: 0
		into: [ :m :el | m + el height ].

	parentBoundsHeight := parentBounds height.
	"parentBoundsHeight := parentBoundsHeight - ((aCollectionOfElements size - 1) * verticalGap) - (2 * verticalOutGap)."
	
	runningIndex := 0.
	parentBounds height > addedHeight
		ifTrue: [ aCollectionOfElements do: [ :element | 
						newHeight := (element height * parentBoundsHeight / addedHeight) asInteger.
						element height: newHeight.
						runningIndex := runningIndex + newHeight + verticalGap ] ]
! !

!RTVerticalLineLayout methodsFor: 'initialize-release' stamp: 'AlexandreBergel 12/14/2013 22:06'!
initialize
	
	super initialize.
	self alignLeft! !

!RTVerticalLineLayout methodsFor: 'hook' stamp: 'AlexandreBergel 12/14/2013 22:06'!
movePointer: pointer accordingToFigure: aNodeFigure
	
	^ pointer setX: pointer x setY: pointer y + aNodeFigure height + self verticalGap! !

!RTVerticalLineLayout methodsFor: 'hook' stamp: 'AlexandreBergel 12/14/2013 22:06'!
positionOriginalPointer: elements 
	| maxWidth delta |
	delta := 0.
	self alignment == #right ifTrue: 
		[ maxWidth := elements maxValue: #width.
		delta := maxWidth ].
	self alignment == #center ifTrue: 
		[ maxWidth := elements maxValue: #width.
		delta := maxWidth / 2.0 ].
	^ ((self paddingLeftFor: elements) + delta) @ (self paddingTopFor: elements)
! !

!RTLayout class methodsFor: 'public' stamp: 'AlexandreBergel 12/14/2013 22:04'!
isAbstract
	^ self name =  #ROLayout! !

!RTLayout class methodsFor: 'public' stamp: 'AlexandreBergel 12/14/2013 22:04'!
isNotAbstract
	^ self isAbstract not! !

!RTLayout class methodsFor: 'public' stamp: 'AlexandreBergel 12/14/2013 22:04'!
on: aCollection
	"main entry point of the class"
	
	"If aCollection is empty, then there is not much to do"
	aCollection isEmpty ifTrue: [ ^ aCollection ].
	self new applyOn: aCollection.
	^ aCollection! !

!RTLayout class methodsFor: 'public' stamp: 'AlexandreBergel 12/14/2013 22:04'!
on: elements edges: edges
	"To make all the layout polymorphic"
	
	^ self on: elements ! !

!RTLayout class methodsFor: 'public' stamp: 'AlexandreBergel 12/14/2013 22:04'!
onElement: aRoassalViewOrRoassalElement

	^ self on: aRoassalViewOrRoassalElement elements! !

!RTLayout class methodsFor: 'public' stamp: 'AlexandreBergel 12/14/2013 22:04'!
onView: aRoassalView

	^ self on: aRoassalView elements! !

!RTLayout methodsFor: 'events' stamp: 'AlexandreBergel 12/14/2013 22:04'!
announce: anEvent
	"trigger an event. Objects who registered to me will get notified"

	| eventToBeSent |
	eventToBeSent := anEvent isBehavior 
						ifTrue: [ anEvent new ]
						ifFalse: [ anEvent ]. 
	
	eventToBeSent layout: self.
	eventHandler announce: eventToBeSent! !

!RTLayout methodsFor: 'hook' stamp: 'AlexandreBergel 12/14/2013 22:04'!
applyOn: elements
	"Return quickly if there is nothing to do"
	elements isEmpty ifTrue: [ ^ self ].
	self executeOnElements: elements! !

!RTLayout methodsFor: 'configuration' stamp: 'AlexandreBergel 12/14/2013 22:04'!
defaultIterationsToSendEvent
	"Every 100 steps an event ROLayoutStep is sent"

	^ 100! !

!RTLayout methodsFor: 'hook' stamp: 'AlexandreBergel 12/14/2013 22:04'!
doExecute: elements
	"Performs the layout"

	self subclassResponsibility! !

!RTLayout methodsFor: 'hook' stamp: 'AlexandreBergel 12/14/2013 22:04'!
doInitialize: elements
	"Method executed before beginning the layout. Useful when the graph to be ordered need to be prepared"
	self announce: (ROLayoutBegin new elements: elements).! !

!RTLayout methodsFor: 'hook' stamp: 'AlexandreBergel 12/14/2013 22:04'!
doPost: elements
	"Method executed after performing the layout"
	self announce: (ROLayoutEnd new elements: elements).! !

!RTLayout methodsFor: 'accessing' stamp: 'AlexandreBergel 12/14/2013 22:04'!
executeOnElements: elements 
	"Execute the layout, myself, on the elements"
		
	maxInterations := elements size.
	self doInitialize: elements.
	self doExecute: elements asOrderedCollection.
	self doPost: elements.
! !

!RTLayout methodsFor: 'initialize-release' stamp: 'AlexandreBergel 12/14/2013 22:17'!
initialize
	translator := ROLayoutTranslator default.
	eventHandler := ROAnnouncer new.
	currentIteraction := 0.
	maxInterations := 0.
	iterationsToSendEvent := self defaultIterationsToSendEvent.
	
	paddingLeft := 5.
	paddingRight := 5.
	paddingTop := 5! !

!RTLayout methodsFor: 'accessing' stamp: 'AlexandreBergel 12/14/2013 22:04'!
iterationsToSendEvent
	"This method return the amount of iterations before sending ROLayoutStep"
	^ iterationsToSendEvent! !

!RTLayout methodsFor: 'accessing' stamp: 'AlexandreBergel 12/14/2013 22:04'!
iterationsToSendEvent: anInteger
	iterationsToSendEvent := anInteger! !

!RTLayout methodsFor: 'hook' stamp: 'AlexandreBergel 12/14/2013 22:04'!
on: elements
	self applyOn: elements! !

!RTLayout methodsFor: 'events' stamp: 'AlexandreBergel 12/14/2013 22:04'!
on: eventClass do: aBlock
	"Register a block as an handler for eventClass"
	
	eventHandler when: eventClass do: aBlock.! !

!RTLayout methodsFor: 'hook' stamp: 'AlexandreBergel 12/14/2013 22:04'!
on: elements edges: edges
	self applyOn: elements! !

!RTLayout methodsFor: 'accessing' stamp: 'AlexandreBergel 12/14/2013 22:14'!
paddingLeft
	^ paddingLeft! !

!RTLayout methodsFor: 'accessing' stamp: 'AlexandreBergel 12/14/2013 22:15'!
paddingLeft: anInteger
	paddingLeft := anInteger! !

!RTLayout methodsFor: 'padding' stamp: 'AlexandreBergel 12/14/2013 22:16'!
paddingLeftFor: elements
	^ paddingLeft! !

!RTLayout methodsFor: 'accessing' stamp: 'AlexandreBergel 12/14/2013 22:14'!
paddingRight
	^ paddingRight! !

!RTLayout methodsFor: 'accessing' stamp: 'AlexandreBergel 12/14/2013 22:16'!
paddingRight: anInteger
	paddingRight := anInteger! !

!RTLayout methodsFor: 'accessing' stamp: 'AlexandreBergel 12/14/2013 22:17'!
paddingTop
	^ paddingTop! !

!RTLayout methodsFor: 'accessing' stamp: 'AlexandreBergel 12/14/2013 22:17'!
paddingTop: anInteger
	paddingTop := anInteger! !

!RTLayout methodsFor: 'padding' stamp: 'AlexandreBergel 12/14/2013 22:17'!
paddingTopFor: elements
	^ paddingTop! !

!RTLayout methodsFor: 'events' stamp: 'AlexandreBergel 12/14/2013 22:04'!
step
	currentIteraction := currentIteraction + 1.
	(currentIteraction \\ self iterationsToSendEvent) = 0 
		ifTrue: [ self announce: (ROLayoutStep new 
											currentIteration: currentIteraction;
											maxInterations: maxInterations) ].
	! !

!RTLayout methodsFor: 'accessing' stamp: 'AlexandreBergel 12/14/2013 22:04'!
translator
	^ translator! !

!RTLayout methodsFor: 'accessing' stamp: 'AlexandreBergel 12/14/2013 22:04'!
translator: t
	translator := t! !

RTObject subclass: #RTShape
	instanceVariableNames: 'color'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Roassal2'!

RTShape subclass: #RTAbstractElementShape
	instanceVariableNames: 'height width'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Roassal2'!

!RTAbstractElementShape methodsFor: 'configuration' stamp: 'AlexandreBergel 12/17/2013 18:13'!
defaultHeight
	^ 5! !

!RTAbstractElementShape methodsFor: 'configuration' stamp: 'AlexandreBergel 12/17/2013 18:13'!
defaultWidth
	^ 5! !

!RTAbstractElementShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/17/2013 18:13'!
height: aValueOrASymbolOrAOneArgBlock
	height := aValueOrASymbolOrAOneArgBlock! !

!RTAbstractElementShape methodsFor: 'rendering' stamp: 'AlexandreBergel 12/17/2013 18:13'!
heightFor: anElement
	^ height rtValue: anElement! !

!RTAbstractElementShape methodsFor: 'initialization' stamp: 'AlexandreBergel 12/17/2013 18:13'!
initialize
	super initialize.
	width := self defaultWidth.
	height := self defaultHeight.
	! !

!RTAbstractElementShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/17/2013 18:13'!
size: aValueOrASymbolOrAOneArgBlock
	self height: aValueOrASymbolOrAOneArgBlock.
	self width: aValueOrASymbolOrAOneArgBlock! !

!RTAbstractElementShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/17/2013 18:13'!
width: aValueOrASymbolOrAOneArgBlock
	width := aValueOrASymbolOrAOneArgBlock! !

!RTAbstractElementShape methodsFor: 'rendering' stamp: 'AlexandreBergel 12/17/2013 18:13'!
widthFor: anElement
	^ width rtValue: anElement! !

RTAbstractElementShape subclass: #RTBox
	instanceVariableNames: 'borderColor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Roassal2'!

!RTBox methodsFor: 'accessing' stamp: 'AlexandreBergel 12/16/2013 09:59'!
borderColor
	^ borderColor! !

!RTBox methodsFor: 'accessing' stamp: 'AlexandreBergel 12/16/2013 09:59'!
borderColor: anObject
	borderColor := anObject! !

!RTBox methodsFor: 'rendering' stamp: 'AlexandreBergel 12/16/2013 16:44'!
renderOn: trachelCanvas for: anElement
	| rec ex |
	ex := (self widthFor: anElement) @ (self heightFor: anElement).
	rec := (anElement position - (ex / 2)) extent: ex.
	^ trachelCanvas buildRectangle: rec color: (self colorFor: anElement)! !

!RTBox methodsFor: 'rendering' stamp: 'AlexandreBergel 12/16/2013 23:22'!
updateFor: anElement
	| rec ex |
	"This is a bit ugly... We need to find a better way"
	anElement trachelShape isNil ifTrue: [ ^ self ].
	ex := (self widthFor: anElement) @ (self heightFor: anElement).
	rec := (anElement position - (ex / 2)) extent: ex.
	anElement trachelShape fromRectangle: rec! !

RTAbstractElementShape subclass: #RTEllipse
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Roassal2'!

!RTEllipse methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/17/2013 18:13'!
initialize
	super initialize.
	color := self defaultColor! !

!RTEllipse methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/17/2013 18:19'!
renderOn: trachelCanvas for: anElement
	| rec ex |
	ex := (self widthFor: anElement) @ (self heightFor: anElement).
	rec := (anElement position - (ex / 2)) extent: ex.
	^ trachelCanvas buildCircleFrom: rec origin to: rec corner color: (self colorFor: anElement)! !

!RTEllipse methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/18/2013 14:00'!
updateFor: anElement
	| rec ex |
	"This is a bit ugly... We need to find a better way"
	anElement trachelShape isNil ifTrue: [ ^ self ].
	ex := (self widthFor: anElement) @ (self heightFor: anElement).
	rec := (anElement position - (ex / 2)) extent: ex.
	anElement trachelShape rectangle: rec; update! !

RTShape subclass: #RTAbstractLine
	instanceVariableNames: 'width'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Roassal2'!

!RTAbstractLine class methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/16/2013 17:40'!
edgeFrom: el1 to: el2
	^ (RTEdge from: el1 to: el2) addShape: self! !

!RTAbstractLine methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/17/2013 16:39'!
updateFor: edge
	edge trachelShape from: edge from position.
	edge trachelShape to: edge to position.
	edge trachelShape update
! !

RTAbstractLine subclass: #RTLine
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Roassal2'!

!RTLine methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/16/2013 17:51'!
renderOn: trachelCanvas for: anEdge
	"w := self widthFor: anEdge."
	^ trachelCanvas buildLineFrom: anEdge from position to: anEdge to position color: (self colorFor: anEdge)! !

RTShape subclass: #RTNull
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Roassal2'!

RTNull class
	instanceVariableNames: 'instance'!

RTNull class
	instanceVariableNames: 'instance'!

!RTNull class methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/14/2013 21:19'!
new
	instance ifNil: [ instance := super new ].
	^ instance! !

!RTNull methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/14/2013 22:27'!
heightFor: element
	^ 0! !

!RTNull methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/14/2013 21:37'!
renderOn: trachelCanvas for: aShapedObject
	"Nothing to do here"! !

!RTNull methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/17/2013 17:20'!
updateFor: anElement
	"Do nothing"! !

!RTNull methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/14/2013 22:27'!
widthFor: element
	^ 0! !

!RTShape class methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/14/2013 21:58'!
element

	^ self instance element! !

!RTShape class methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/14/2013 21:58'!
elementOn: aModel

	^ self instance elementOn: aModel! !

!RTShape class methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/14/2013 21:42'!
instance
	^ self new! !

!RTShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/16/2013 17:52'!
color
	^ color! !

!RTShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/16/2013 17:52'!
color: anObject
	color := anObject! !

!RTShape methodsFor: 'rendering' stamp: 'AlexandreBergel 12/16/2013 17:52'!
colorFor: anElement
	^ color rtValue: anElement! !

!RTShape methodsFor: 'configuration' stamp: 'AlexandreBergel 12/16/2013 16:18'!
defaultColor

	^ Color veryVeryLightGray! !

!RTShape methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/14/2013 21:57'!
element

	^ RTElement new addShape: self; yourself! !

!RTShape methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/14/2013 21:57'!
elementOn: aModel

	^ RTElement new model: aModel; addShape: self; yourself! !

!RTShape methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/16/2013 18:06'!
initialize
	super initialize.
	color := self defaultColor! !

!RTShape methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/14/2013 21:42'!
instance
	^ self! !

!RTShape methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/16/2013 18:06'!
renderOn: trachelCanvas for: aShapedObject
	self subclassResponsibility! !

RTObject subclass: #RTShapedObject
	instanceVariableNames: 'shape announcer trachelShape'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Roassal2'!

RTShapedObject subclass: #RTEdge
	instanceVariableNames: 'from to'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Roassal2'!

!RTEdge class methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/16/2013 17:43'!
from: fromEl to: toEl
	^ self new from: fromEl; to: toEl; yourself! !

!RTEdge methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/16/2013 17:45'!
addShape: aRTShape
	shape := aRTShape instance! !

!RTEdge methodsFor: 'accessing' stamp: 'AlexandreBergel 12/16/2013 17:44'!
from
	
	^ from! !

!RTEdge methodsFor: 'accessing' stamp: 'AlexandreBergel 12/16/2013 19:05'!
from: anElement
	
	from := anElement.
	anElement addOutgoingEdge: self! !

!RTEdge methodsFor: 'accessing' stamp: 'AlexandreBergel 12/16/2013 17:44'!
to
	
	^ to! !

!RTEdge methodsFor: 'accessing' stamp: 'AlexandreBergel 12/16/2013 19:06'!
to: anElement
	
	to := anElement.
	anElement addIncomingEdge: self! !

RTShapedObject subclass: #RTElement
	instanceVariableNames: 'position model width height incomingEdges outgoingEdges'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Roassal2'!

!RTElement class methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/14/2013 21:10'!
on: anObject

	^ self new model: anObject; yourself! !

!RTElement methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/16/2013 19:04'!
addIncomingEdge: anEdge
	"Method invoked by the class Edge, the user is not supposed to call this method directly.
	We keep a list of incoming and outoing edges because when the element is dragged and dropped, 
	the edges may have to be updated"
	incomingEdges isNil ifTrue: [ incomingEdges := OrderedCollection new ]. 
	incomingEdges add: anEdge! !

!RTElement methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/16/2013 19:05'!
addOutgoingEdge: anEdge
	"Method invoked by the class Edge, the user is not supposed to call this method directly.
	We keep a list of incoming and outoing edges because when the element is dragged and dropped, 
	the edges may have to be updated"
	outgoingEdges isNil ifTrue: [ outgoingEdges := OrderedCollection new ]. 
	outgoingEdges add: anEdge! !

!RTElement methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/16/2013 13:02'!
addShape: aRTShape

	"I am not sure we need to have aRTShape here. It should instead be a trachel shape"
	shape := aRTShape instance.
	"shape announcer: self announcer."

	width := width max: (shape widthFor: self).
	height := height max: (shape heightFor: self).
	
	! !

!RTElement methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/14/2013 22:25'!
height
	^ height! !

!RTElement methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/16/2013 19:02'!
incomingEdges
	incomingEdges isNil ifTrue: [ ^ #() ].
	^ incomingEdges! !

!RTElement methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/14/2013 22:25'!
initialize
	super initialize.
	position := 0 @ 0.
	width := 0.
	height := 0! !

!RTElement methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/14/2013 21:09'!
model
	^ model! !

!RTElement methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/14/2013 21:09'!
model: anObject
	"anObject could be any Smalltalk object. Shapes are computed based on that object"
	model := anObject! !

!RTElement methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/16/2013 19:02'!
outgoingEdges
	outgoingEdges isNil ifTrue: [ ^ #() ].
	^ outgoingEdges! !

!RTElement methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/14/2013 21:06'!
position
	^ position! !

!RTElement methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/14/2013 21:06'!
translateBy: delta
	self translateTo: position + delta! !

!RTElement methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/16/2013 23:16'!
translateTo: newPosition
	position := newPosition.
	self outgoingEdges do: [ :edge | edge update ].
	self incomingEdges do: [ :edge | edge update ].
	self update! !

!RTElement methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/14/2013 22:25'!
width
	^ width! !

!RTShapedObject methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/15/2013 23:37'!
@ anInteractionClassOrInstance
	"Add an interaction to the node"
	
	self addInteraction: anInteractionClassOrInstance! !

!RTShapedObject methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/15/2013 23:40'!
addInteraction: anInteractionClassOrInstance
 	anInteractionClassOrInstance initializeElement: self! !

!RTShapedObject methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/16/2013 10:00'!
announce: evt
	self announcer announce: evt! !

!RTShapedObject methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/16/2013 09:50'!
announcer
	announcer ifNil: [ announcer := Announcer new ].
	^ announcer! !

!RTShapedObject methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/14/2013 21:21'!
initialize
	super initialize.
	shape := RTNull new! !

!RTShapedObject methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/16/2013 09:50'!
on: evt do: aBlock
	self announcer on: evt do: aBlock! !

!RTShapedObject methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/16/2013 22:53'!
renderOn: trachelCanvas

	trachelShape := shape renderOn: trachelCanvas for: self.
	trachelShape announcer: self announcer.
	trachelShape element: self! !

!RTShapedObject methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/14/2013 21:28'!
shape
	^ shape! !

!RTShapedObject methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/16/2013 12:58'!
trachelShape
	^ trachelShape! !

!RTShapedObject methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/16/2013 23:06'!
update
	"Update the path of the trachel shape. This has the effect of moving the visual widget"
	shape updateFor: self
! !

RTObject subclass: #RTView
	instanceVariableNames: 'elements trachelCanvas'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Roassal2'!

!RTView methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/16/2013 11:16'!
add: anElement
	elements add: anElement.
	"anElement shape renderOn: trachelCanvas for: anElement"! !

!RTView methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/16/2013 11:20'!
createAndFillCanvas
	| trCanvas |
	trCanvas := TRCanvas new.
	elements do: [ :el | el renderOn: trCanvas ].
	^ trCanvas! !

!RTView methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/14/2013 21:59'!
elements
	^ elements! !

!RTView methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/16/2013 11:16'!
initialize
	super initialize.
	elements := OrderedCollection new.
	"trachelCanvas := TRCanvas new."! !

!RTView methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/14/2013 20:36'!
numberOfElements
	^ elements size! !

!RTView methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/16/2013 11:20'!
open
	"^ trachelCanvas open"

	^ self createAndFillCanvas open! !
